/*
 * Copyright (C) 1997-2004 Andre Noll <noll@mathematik.tu-darmstadt.de>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "para.h"
#include "server.cmdline.h"

/* status flags */
#define AFS_NOMORE 1
#define AFS_NEXT 2
#define AFS_REPOS 4
#define AFS_PLAYING 8
#define AFS_REREAD_DB_INFO 16
#define DBT_CHANGE 32

#define MMD_INFO_SIZE 16384
/* contains a list of all valid commands */
struct linked_cmd_list {
        struct linked_cmd_list *next;
	struct command *cmd;
};

/* data needed to authenticate user */
struct user{
	char name[MAXLINE];
	char pubkey_file[_POSIX_PATH_MAX];
	unsigned int perms;
};

/* contains data for arbitrary command */
struct command {
	char *name;
	unsigned int perms;
	char *description;
	char *usage;
	char *help;
	int (*handler)(int, int, char **);
};

struct audio_format {
	const char *name;
	int (*init)(void*);
	char *open_cmd;
	FILE *sw;
	pid_t sw_pid;
	void (*end_of_input)(FILE*, FILE**, pid_t*);
	void (*close_audio_file)(FILE*);
	int (*get_file_info)(FILE*, char*, long unsigned*, int*);
	int (*reposition_stream)(FILE *, long unsigned, pid_t *);
	int (*read_chunk)(FILE*);
	int (*write_chunk)(FILE *, char*, long unsigned*);
};

struct dbtool {
	const char *name;
	struct command *cmd_list;
	char **(*get_song_list)(int);
	void  (*update_song)(char *);
	int (*init)(struct dbtool *);
	void (*shutdown)(void);
};

/* there's one struct of this type which lives in shared memory
 * for communication between the server instances. Access to this
 * area is serialized via mmd_lock()/mmd_unlock(). There are two 
 * reasons for a variable to be included here:
 * (a) at least one command (i.e. child of the server) must be able to 
 *     change its value
 * or
 * (b) The contents are listed in the stat command and have to be up to date  
 */
struct misc_meta_data{
/* audio file sender info */ 
	long unsigned int size;
	char filename[_POSIX_PATH_MAX];
	time_t mtime;
	unsigned int afs_status_flags;
	long unsigned chunks_sent, chunks_total;
	int seconds_total;
	char audio_file_info[MMD_INFO_SIZE];
	int audio_format;
/* server info */
	unsigned int num_played;
	unsigned int num_commands;
	unsigned int num_connects;
	unsigned int active_connections;
	pid_t server_pid;
/* dbtool */
	char dbinfo[MMD_INFO_SIZE];
	int dbt_num; /* number of current dbtool */
	int dbt_change; /* requests dbtool change if non-negative */
};

int afs_init(struct gengetopt_args_info *);
int afs_send_chunk(FILE*);
FILE *afs_preselect(void);
char *afs_supported_audio_formats(void);
const char *audio_format_name(int);

int mysql_dbtool_init(struct dbtool*);
int dopey_dbtool_init(struct dbtool*);

int server_uptime (int );

void clog(int, char* ,...);
int check_perms(unsigned int, struct command*);

struct command *parse_cmd(char*);

void mmd_unlock(void);
void mmd_lock(void);
